#! /usr/bin/env python
# coding: utf-8

import json
import MySQLdb
import threading
from DBUtils.PersistentDB import PersistentDB
from DBUtils.PooledDB import PooledDB

__author__ = '鹛桑够'


class SimpleDB(object):
    TIME_FORMAT = '%Y-%m-%d %H:%M:%S'
    DATE_FORMAT = '%Y-%m-%d'
    AUTO_CLOSE = False

    _pool = dict()
    max_connections = 3

    def __init__(self, host, port, user, password, db_name, charset="utf8"):
        self.thread_data = threading.local()
        self.thread_data.conn = None
        self.thread_data.cursor = None
        self.host = host
        self._db_port = port
        self._db_user = user
        self._db_password = password
        self._db_name = db_name
        if charset == "":
            charset = None
        self._connect_charset = charset

    @staticmethod
    def _connect(host, port, user, password, db_name, connect_charset=None, shareable=True):
        _pool_id = "%s_%s_%s_%s" % (host, port, user, db_name)
        if _pool_id not in SimpleDB._pool:
            comm_kwargs = dict(host=host, port=port, user=user, passwd=password, db=db_name)
            if connect_charset is not None:
                comm_kwargs["charset"] = connect_charset
            if shareable is True:
                pool = PooledDB(MySQLdb, blocking=1, maxconnections=SimpleDB.max_connections, **comm_kwargs)
            else:
                pool = PersistentDB(MySQLdb, **comm_kwargs)
            SimpleDB._pool[_pool_id] = pool

        conn = SimpleDB._pool[_pool_id].connection()
        cursor = conn.cursor()
        return conn, cursor

    def connect(self):
        conn, cursor = self._connect(self.host, self._db_port, self._db_user, self._db_password, self._db_name,
                                     self._connect_charset)
        self.thread_data.conn = conn
        self.thread_data.cursor = cursor
        return conn, cursor

    def literal(self, s):
        if isinstance(s, basestring):
            pass
        elif isinstance(s, dict) or isinstance(s, tuple) or isinstance(s, list):
            s = json.dumps(s)
        elif isinstance(s, set):
            s = json.dumps(list(s))
        return s

    @property
    def current_transaction(self):
        if "exec_transaction" not in self.thread_data.__dict__:
            self.thread_data.exec_transaction = None
        if self.thread_data.exec_transaction is not True:
            return False
        return True

    @current_transaction.setter
    def current_transaction(self, value):
        self.thread_data.exec_transaction = value

    def start_transaction(self):
        if self.current_transaction is False:
            self.current_transaction = True

    def end_transaction(self, **kwargs):
        fail = kwargs.pop("fail", False)
        if fail is True and self.thread_data.conn:
            self.thread_data.conn.rollback()
        self.current_transaction = False
        self.close()

    def execute(self, sql_query, args=None, freq=0, print_sql=False, w_literal=False, auto_close=AUTO_CLOSE):
        if "cursor" not in self.thread_data.__dict__:
            self.thread_data.cursor = None
        if self.thread_data.cursor is None:
            self.connect()
        if args is not None and w_literal is False:
            if isinstance(args, (tuple, list)) is True:
                args = map(self.literal, args)
            elif isinstance(args, dict) is True:
                for k, v in args.items():
                    args[k] = self.literal(v)
        try:
            if print_sql is True:
                print(sql_query)
            handled_item = self.thread_data.cursor.execute(sql_query, args=args)
        except MySQLdb.Error as error:
            if self.current_transaction is True:
                self.end_transaction(fail=True)
                self.close()
                raise MySQLdb.Error(error)
            print(error)
            if freq >= 3 or error.args[0] in [1054, 1064, 1146, 1065, 1040]:  # 列不存在 sql错误 表不存在 empty_query too_many_connectons
                raise MySQLdb.Error(error)
            self.connect()
            return self.execute(sql_query=sql_query, args=args, freq=freq + 1, w_literal=True, auto_close=auto_close)
        if auto_close is True and self.current_transaction is False:
            self.close()
        elif self.current_transaction is False:
            self.thread_data.conn.commit()
        return handled_item

    def fetchone(self):
        r = self.thread_data.cursor.fetchone()
        if self.current_transaction is False:
            self.close()
        return r

    def fetchall(self):
        r = self.thread_data.cursor.fetchall()
        if self.current_transaction is False:
            self.close()
        return r

    def close(self):
        if self.thread_data.cursor:
            self.thread_data.cursor.close()
        if self.thread_data.conn:
            self.thread_data.conn.commit()
            self.thread_data.conn.close()
        self.thread_data.conn = None
        self.thread_data.cursor = None




#! /usr/bin/env python
# coding: utf-8

import os
import sys
import json
from _conf_db import ConfDB
from _execute import SelectDB

__author__ = '鹛桑够'


class TableDB(ConfDB, SelectDB):
    t_tables = "information_schema.TABLES"
    t_columns = "information_schema.columns"

    def __init__(self, conf_path=None, conf_dir=None, readonly=False, user=None, password=None):
        ConfDB.__init__(self, conf_path, conf_dir, readonly, user, password)

    def table_exist(self, t_name):
        where_value = dict(TABLE_SCHEMA=self._db_name, TABLE_TYPE='BASE TABLE', TABLE_NAME=t_name)
        cols = ["TABLE_NAME", "CREATE_TIME", "TABLE_COMMENT"]
        items = self.execute_select(self.t_tables, where_value=where_value, cols=cols, package=True)
        l = len(items)
        if l == 0:
            return False
        return True

    def get_table_info(self, table_name):
        cols = ["column_name",  "column_type", "column_key", "column_default", "extra", "column_comment", "is_nullable"]
        where_value = dict(table_name=table_name, table_schema=self._db_name)
        items = self.execute_select(self.t_columns, cols=cols, where_value=where_value)
        return items

    def add_table_column(self, t_name, col_name, col_type, col_comment='', allow_null=False, default_value=None,
                         uni_key=False):
        add_sql = u"ALTER TABLE {t_name} ADD COLUMN  {c_name} {c_type} {c_dv} {c_null} COMMENT '{c_comment}';"
        c_null = "" if allow_null is True else "NOT NULL"
        c_dv = "" if default_value is None else "DEFAULT %s" % default_value
        sql = add_sql.format(t_name=t_name, c_name=col_name, c_type=col_type, c_null=c_null, c_comment=col_comment,
                             c_dv=c_dv)
        self.execute(sql, print_sql=True)
        if uni_key is True:
            uni_sql = "ALTER TABLE {t_name} ADD UNIQUE ({col_name});".format(t_name=t_name, col_name=col_name)
            self.execute(uni_sql, print_sql=True)
        return True

    def update_table(self, table_desc):
        del_msg = "Table {table_name} Column {col_name} Not Exist In Latest Table Structure, Delete It Please Exec:\n"
        del_msg += "ALTER TABLE {table_name} DROP COLUMN {col_name};\n"

        t_name = table_desc["table_name"]
        cols_items = self.get_table_info(t_name)
        for desc_item in table_desc["table_cols"]:
            is_exist = False
            for i in range(len(cols_items) - 1, -1, -1):
                cols_item = cols_items[i]
                if cols_item["column_name"].lower() == desc_item["col_name"].lower():
                    is_exist = True

                    cols_items.remove(cols_item)
            if is_exist is False:
                desc_item["allow_null"] = True
                self.add_table_column(t_name, **desc_item)
                print("%s not exist" % desc_item["col_name"])
        if len(cols_items) > 0:
            for item in cols_items:
                sys.stderr.write(del_msg.format(table_name=t_name, col_name=item["column_name"]))
        return True

    def create_table(self, table_desc):
        if type(table_desc) != dict:
            return False, "table_desc need dict"
        if "table_name" not in table_desc or "table_cols" not in table_desc:
            return False, "Bad table_desc, need table_name and table_cols"
        exists_drop = table_desc.get("exists_drop", False)
        table_cols = table_desc["table_cols"]
        primary_key = []
        uni_key = []
        mul_key = []
        fields = []
        for col in table_cols:
            field_sql = ""
            if "col_name" not in col or "col_type" not in col:
                return False, "Bad col, need col_name and col_type"
            col_name = col["col_name"]
            field_sql += "%s %s" % (col_name, col["col_type"])
            if "allow_null" in col and col["allow_null"] is True:
                pass
            else:
                field_sql += " NOT NULL"
            if "auto_increment" in col and col["auto_increment"] is True:
                field_sql += " auto_increment"
                primary_key.append(col_name)
            elif "pri_key" in col and col["pri_key"] is True:
                primary_key.append(col_name)
            if "uni_key" in col and col["uni_key"] is True:
                uni_key.append(col_name)
            if "mul_key" in col and col["mul_key"] is True:
                mul_key.append(col_name)
            if "default_value" in col and col["default_value"] is not None:
                "".startswith("int")
                if col["col_type"].startswith("int") or col["col_type"].startswith("tinyint") or col["col_type"].startswith("bit"):
                    field_sql += " default %s" % col["default_value"]
                else:
                    field_sql += " default '%s'" % col["default_value"]

            if "col_comment" in col and col["col_comment"] is not None:
                field_sql += " COMMENT '%s'" % col["col_comment"]
            fields.append(field_sql)
        table_name = table_desc["table_name"]
        if len(primary_key) > 0:
            fields.append(" PRIMARY KEY (%s)" % ",".join(primary_key))
        if len(uni_key) > 0:
            for key in uni_key:
                fields.append(" UNIQUE KEY ({0})".format(key))
        if len(mul_key) > 0:
            for key in mul_key:
                fields.append(" INDEX {0} ({0})".format(key))
        execute_message = ""
        if "table_suffix" in table_desc and isinstance(table_desc["table_suffix"], list):
            if "table_comment" in table_desc and table_desc["table_comment"] != "":
                comment = table_desc["table_comment"]
            else:
                comment = "%s"
            for t_item in table_desc["table_suffix"]:
                t_name_item = "%s_%s" % (table_name, t_item)
                if self.table_exist(t_name_item) is True:
                    if exists_drop is False:
                        execute_message += "%s Table exist\n" % t_name_item
                        continue
                    print("drop")
                    self.execute("DROP TABLE %s" % t_name_item, auto_close=True)
                create_table_sql = "CREATE TABLE %s ( %s )" % (t_name_item, ",".join(fields))
                item_comment = comment % t_item
                create_table_sql += " COMMENT '%s'" % item_comment
                create_table_sql += " DEFAULT CHARSET=utf8;"
                self.execute(create_table_sql, auto_close=True)
                execute_message += "CREATE TABLE %s Success \n" % t_name_item
        else:
            if self.table_exist(table_name) is True:
                if exists_drop is False:
                    self.update_table(table_desc)
                    return False, "Table exist."
                self.execute("DROP TABLE %s" % table_name, auto_close=True)
            create_table_sql = "CREATE TABLE %s ( %s )" % (table_name, ",".join(fields))
            if "table_comment" in table_desc and table_desc["table_comment"] != "":
                create_table_sql += " COMMENT '%s'" % table_desc["table_comment"]
            create_table_sql += " DEFAULT CHARSET=utf8;"
            self.execute(create_table_sql, auto_close=True)
            execute_message += "CREATE TABLE %s Success \n" % table_name
        return True, execute_message

    def create_mul_table(self, mul_table_desc):
        if type(mul_table_desc) != list:
            return False, "mul_table_desc need list"
        mul_len = len(mul_table_desc)
        if mul_len <= 0:
            return False, "mul_table_desc len is 0"
        fail_index = []
        for i in range(mul_len):
            table_desc = mul_table_desc[i]
            result, message = self.create_table(table_desc)
            if result is False:
                fail_index.append((i, message))
        return True, fail_index

    def create_from_json_file(self, json_file):
        if os.path.isfile(json_file) is False:
            print("json file not exist")
            return False, "json file not exist"
        read_json = open(json_file)
        json_content = read_json.read()
        read_json.close()
        try:
            json_desc = json.loads(json_content, "utf-8")
        except ValueError as ve:
            print(ve)
            return False, "File content not json"
        result, message = self.create_mul_table(json_desc)
        return result, message

    def create_from_dir(self, desc_dir):
        if os.path.isdir(desc_dir) is False:
            return False, "desc dir not exist"
        desc_files = os.listdir(desc_dir)
        create_info = []
        for item in desc_files:
            if not item.endswith(".json"):
                continue
            file_path = desc_dir+ "/" + item
            result, info = self.create_from_json_file(file_path)
            create_info.append({"file_path": file_path, "create_result": result, "create_info": info})
        return True, create_info



#! /usr/bin/env python
# coding: utf-8

import os
from _conf_db import ConfDB
from _execute import SelectDB, InsertDB, UpdateDB, DeleteDB


__author__ = '鹛桑够'


class DB(ConfDB, SelectDB, InsertDB, UpdateDB, DeleteDB):
    t_tables = "information_schema.TABLES"

    def __init__(self, conf_path=None, conf_dir=None, readonly=False, user=None, password=None):
        ConfDB.__init__(self, conf_path, conf_dir, readonly, user, password)

    def execute_call(self, p_name, *args):
        sql_query = "CALL %s(" % p_name
        sql_query += ",".join(map(self.literal, args))
        sql_query += ");"
        return self.execute(sql_query)

    def table_exist(self, t_name):
        where_value = dict(TABLE_SCHEMA=self._db_name, TABLE_TYPE='BASE TABLE', TABLE_NAME=t_name)
        cols = ["TABLE_NAME", "CREATE_TIME", "TABLE_COMMENT"]
        l = self.execute_select(self.t_tables, where_value=where_value, cols=cols, package=False)
        if l == 0:
            return False
        return True

    def create_user(self, user, password, host='localhost', db=None, readonly=False):
        l = self.execute_select("mysql.user", where_value=dict(user=user, host=host))
        if l <= 0:
            c_sql = "CREATE USER %s@%s IDENTIFIED BY %s;"
            self.execute(c_sql, args=(user, host, password))
        if db is not None:
            if readonly is False:
                g_sql = "GRANT ALL ON {db}.* TO %s@%s;"
            else:
                g_sql = "GRANT SELECT ON {db}.* TO %s@%s;"
            self.execute(g_sql.format(db=db), args=(user, host))
        return True

    def root_init_conf(self, host='localhost'):
        o = self._read_conf(self.conf_path, False)
        self.create_user(o["user"], o["password"], host=host, db=o["db_name"], readonly=False)
        o = self._read_conf(self.conf_path, True)
        self.create_user(o["user"], o["password"], host=host, db=o["db_name"], readonly=True)

    def source_file(self, file_path):
        cmd = "mysql -u%s -p%s %s < %s" % (self._db_user, self._db_password, self._db_name, file_path)
        os.system(cmd)



#! /usr/bin/env python
# coding: utf-8

import json
import MySQLdb

__author__ = '鹛桑够'


class SimpleDB(object):
    TIME_FORMAT = '%Y-%m-%d %H:%M:%S'
    DATE_FORMAT = '%Y-%m-%d'

    def __init__(self, host, port, user, password, db_name):
        self.conn = None
        self.cursor = None
        self.host = host
        self._db_port = port
        self._db_user = user
        self._db_password = password
        self._db_name = db_name

    @staticmethod
    def _connect(host, port, user, password, db_name):
        conn = MySQLdb.connect(host=host, port=port, user=user, passwd=password, db=db_name, charset='utf8')
        cursor = conn.cursor()
        conn.autocommit(True)
        return conn, cursor

    def connect(self):
        self.conn, self.cursor = self._connect(self.host, self._db_port, self._db_user, self._db_password,
                                               self._db_name)

    def literal(self, s):
        if not self.conn:
            self.connect()
        if isinstance(s, basestring):
            pass
        elif isinstance(s, dict) or isinstance(s, tuple) or isinstance(s, list):
            s = json.dumps(s)
        elif isinstance(s, set):
            s = json.dumps(list(s))
        s = self.conn.literal(s)
        return s

    def execute(self, sql_query, args=None, freq=0, print_sql=False):
        if self.cursor is None:
            self.connect()
        if isinstance(sql_query, unicode):
            sql_query = sql_query.encode(self.conn.unicode_literal.charset)
        if args is not None:
            if isinstance(args, dict):
                sql_query = sql_query % dict((key, self.literal(item)) for key, item in args.iteritems())
            else:
                sql_query = sql_query % tuple([self.literal(item) for item in args])
        try:
            if print_sql is True:
                print(sql_query)
            handled_item = self.cursor.execute(sql_query)
        except MySQLdb.Error as error:
            print(error)
            if freq >= 3 or error.args[0] in [1054, 1064, 1146, 1065]:  # 列不存在 sql错误 表不存在 empty_query
                raise MySQLdb.Error(error)
            self.connect()
            return self.execute(sql_query=sql_query, freq=freq + 1)
        return handled_item

    def fetchone(self):
        return self.cursor.fetchone()

    def fetchall(self):
        return self.cursor.fetchall()

    def close(self):
        if self.cursor:
            self.cursor.close()
        self.conn.close()


#! /usr/bin/env python
# coding: utf-8

from datetime import datetime, date
from _db import SimpleDB

__author__ = '鹛桑够'


def merge_where(where_value=None, where_is_none=None, where_cond=None, where_cond_args=None, prefix_value=None):
    args = []
    if where_cond is None:
        where_cond = list()
    else:
        where_cond = list(where_cond)
        if isinstance(where_cond_args, (list, tuple)):
            args.extend(where_cond_args)
    if where_value is not None:
        where_args = dict(where_value).values()
        args.extend(where_args)
        for key in dict(where_value).keys():
            where_cond.append("%s=%%s" % key)
    if isinstance(prefix_value, dict) is True:
        for key in prefix_value.keys():
            v = "%s" % prefix_value[key]
            v = v.replace("_", r"\_").replace("%", r"\%")
            v = "%s%%" % v
            where_cond.append("%s LIKE %%s" % key)
            args.append(v)
    if where_is_none is not None and len(where_is_none) > 0:
        for key in where_is_none:
            where_cond.append("%s is NULL" % key)
    return where_cond, args


class SelectDB(SimpleDB):

    def execute_multi_select(self, table_name, where_value=None, cols=None, package=True, **kwargs):
        kwargs = dict(kwargs)
        if cols is None:
            select_item = "*"
        else:
            select_item = ",".join(tuple(cols))
        select_sql = "SELECT %s FROM %s" % (select_item, table_name)
        sql_query = ""
        args = []
        if isinstance(where_value, dict):
            for item in where_value:
                value_list = where_value[item]
                for value in value_list:
                    sql_query += "{0} WHERE {1}=%s union ".format(select_sql, item)
                    args.append(value)
        if len(sql_query) <= 0:
            return []
        sql_query = sql_query[:-7]
        order_by = kwargs.pop("order_by", None)
        order_desc = kwargs.pop("order_desc", False)
        if order_by is not None:
            if isinstance(order_by, list) or isinstance(order_by, tuple):
                sql_query += " ORDER BY %s" % ",".join(order_by)
            elif isinstance(order_by, unicode) or isinstance(order_by, str):
                sql_query += " ORDER BY %s" % order_by
            if order_desc is True:
                sql_query += " DESC"
        sql_query += ";"
        exec_result = self.execute(sql_query, args, auto_close=False)
        db_items = self.fetchall()
        if cols is not None and package is True:
            select_items = []
            for db_item in db_items:
                r_item = dict()
                for i in range(len(cols)):
                    c_v = db_item[i]
                    if isinstance(c_v, datetime):
                        c_v = c_v.strftime(self.TIME_FORMAT)
                    elif isinstance(c_v, date):
                        c_v = c_v.strftime(self.DATE_FORMAT)
                    elif isinstance(c_v, str):
                        if c_v == "\x00":
                            c_v = False
                        elif c_v == "\x01":
                            c_v = True
                        else:
                            print(c_v)
                    r_item[cols[i]] = c_v
                select_items.append(r_item)
            return select_items
        return db_items

    def execute_select(self, table_name, where_value=None, where_cond=None, cols=None, package=True, **kwargs):
        kwargs = dict(kwargs)
        where_is_none = kwargs.pop("where_is_none", None)
        where_cond_args = kwargs.pop("where_cond_args", None)
        prefix_value = kwargs.pop("prefix_value", None)
        update_lock = kwargs.pop("update_lock", False)
        where_cond, args = merge_where(where_value=where_value, where_cond=where_cond, where_is_none=where_is_none,
                                       where_cond_args=where_cond_args, prefix_value=prefix_value)
        if cols is None:
            select_item = "*"
        else:
            select_item = ",".join(tuple(cols))
        if len(where_cond) > 0:
            sql_query = "SELECT %s FROM %s WHERE %s" % (select_item, table_name, " AND ".join(where_cond))
        else:
            sql_query = "SELECT %s FROM %s" % (select_item, table_name)
        order_by = kwargs.pop("order_by", None)
        order_desc = kwargs.pop("order_desc", False)
        limit = kwargs.pop("limit", None)
        if order_by is not None:
            if isinstance(order_by, list) or isinstance(order_by, tuple):
                sql_query += " ORDER BY %s" % ",".join(order_by)
            elif isinstance(order_by, unicode) or isinstance(order_by, str):
                sql_query += " ORDER BY %s" % order_by
            if order_desc is True:
                sql_query += " DESC"
        if isinstance(limit, int):
            sql_query += " LIMIT %s" % limit
        if update_lock is True:
            sql_query += " FOR UPDATE"
        sql_query += ";"
        print_sql = kwargs.get("print_sql", False)
        exec_result = self.execute(sql_query, args, print_sql=print_sql, auto_close=False)
        db_items = self.fetchall()
        if cols is not None and package is True:
            select_items = []
            for db_item in db_items:
                r_item = dict()
                for i in range(len(cols)):
                    c_v = db_item[i]
                    if isinstance(c_v, datetime):
                        c_v = c_v.strftime(self.TIME_FORMAT)
                    elif isinstance(c_v, date):
                        c_v = c_v.strftime(self.DATE_FORMAT)
                    elif isinstance(c_v, str):
                        if c_v == "\x00":
                            c_v = False
                        elif c_v == "\x01":
                            c_v = True
                        else:
                            print(c_v)
                    r_item[cols[i]] = c_v
                select_items.append(r_item)
            return select_items
        return db_items

    def execute_select_with_lock(self, table_name, where_value=None, where_cond=None, cols=None, package=True, **kwargs):
        kwargs["update_lock"] = True
        return self.execute_select(table_name, where_value=where_value, where_cond=where_cond, cols=cols,
                                   package=package, **kwargs)

    def execute_select_left(self, b_t_name, j_t_name, join_key, **kwargs):
        where_is_none = kwargs.pop("where_is_none", None)
        where_value = kwargs.pop("where_value", {"1": 1})
        where_cond = kwargs.pop("where_cond", None)
        where_cond, args = merge_where(where_value=where_value, where_cond=where_cond, where_is_none=where_is_none)
        cols = list()
        package_keys = list()
        b_cols = kwargs.pop("b_cols", None)
        if b_cols is not None and isinstance(b_cols, list):
            for b_col in b_cols:
                cols.append("%s.%s" % (b_t_name, b_col))
                package_keys.append(b_col)
        j_cols = kwargs.pop("j_cols", None)
        if j_cols is not None and isinstance(j_cols, list):
            for j_col in j_cols:
                cols.append("%s.%s" % (j_t_name, j_col))
                package_keys.append(j_col)
        if len(cols) <= 0:
            s_item = "*"
        else:
            s_item = ",".join(cols)
        sql_query = "SELECT {0} FROM {1} LEFT JOIN {2} ON {1}.{3}={2}.{3}".format(s_item, b_t_name, j_t_name, join_key)
        if len(where_cond) > 0:
            sql_query += " WHERE %s" % " AND ".join(where_cond)
        order_by = kwargs.pop("order_by", None)
        order_desc = kwargs.pop("order_desc", False)
        limit = kwargs.pop("limit", None)
        if order_by is not None and (isinstance(order_by, list) or isinstance(order_by, tuple)):
            sql_query += " ORDER BY %s" % ",".join(order_by)
            if order_desc is True:
                sql_query += " DESC"
        if isinstance(limit, int):
            sql_query += " LIMIT %s" % limit
        sql_query += ";"
        exec_result = self.execute(sql_query, args, auto_close=False)
        db_items = self.fetchall()
        if len(package_keys) > 0:
            select_items = []
            for db_item in db_items:
                r_item = dict()
                for i in range(len(package_keys)):
                    c_v = db_item[i]
                    if isinstance(c_v, datetime):
                        c_v = c_v.strftime(self.TIME_FORMAT)
                    elif isinstance(c_v, date):
                        c_v = c_v.strftime(self.DATE_FORMAT)
                    elif isinstance(c_v, str):
                        if c_v == "\x00":
                            c_v = False
                        elif c_v == "\x01":
                            c_v = True
                        else:
                            print(c_v)
                    r_item[package_keys[i]] = c_v
                select_items.append(r_item)
            return select_items
        return db_items


class InsertDB(SimpleDB):

    def execute_insert(self, table_name, kwargs, ignore=False):
        keys = dict(kwargs).keys()
        if ignore is True:
            sql_query = "INSERT IGNORE INTO %s (%s) VALUES (%%(%s)s);" % (table_name, ",".join(keys),
                                                                          ")s,%(".join(keys))
        else:
            sql_query = "INSERT INTO %s (%s) VALUES (%%(%s)s);" % (table_name, ",".join(keys), ")s,%(".join(keys))
        return self.execute(sql_query, args=kwargs, auto_close=True)

    def execute_duplicate_insert(self, t_name, kwargs, u_keys=None, p1_keys=None, u_v=None):
        if isinstance(kwargs, dict) is False:
            raise TypeError()
        if u_v is None:
            u_v = []
        if isinstance(u_v, list) is False:
            raise TypeError()
        keys = kwargs.keys()
        if isinstance(u_keys, (tuple, list)) is True:
            u_v.extend(map(lambda x: "{0}=VALUES({0})".format(x), u_keys))
        if isinstance(p1_keys, (tuple, list)) is True:
            u_v.extend(map(lambda x: "{0}={0}+1".format(x), p1_keys))
        if len(u_v) <= 0:
            return self.execute_insert(t_name, kwargs)
        sql = "INSERT INTO %s (%s) VALUES (%%(%s)s) ON DUPLICATE KEY UPDATE %s;" \
              % (t_name, ",".join(keys), ")s,%(".join(keys), ",".join(u_v))
        return self.execute(sql, args=kwargs, auto_close=True)

    def execute_insert_mul(self, table_name, cols, value_list, ignore=False):
        keys = cols
        if ignore is True:
            sql_query = "INSERT IGNORE INTO %s (%s) VALUES " % (table_name, ",".join(keys))
        else:
            sql_query = "INSERT INTO %s (%s) VALUES " % (table_name, ",".join(keys))
        if not isinstance(value_list, list):
            return -1
        if len(value_list) <= 0:
            return 0
        args = []
        for value_item in value_list:
            sql_query += "(" + ("%s," * len(value_item)).rstrip(",") + "),"
            args.extend(value_item)
        sql_query = sql_query.rstrip(",") + ";"
        return self.execute(sql_query, args=args, auto_close=True)

    def execute_duplicate_insert_mul(self, t_name, cols, values, u_keys=None, p1_keys=None, concat_keys=None, u_v=None,
                                     print_sql=False):
        u_l = list()
        if isinstance(u_v, (tuple, list)) is True:
            u_l.extend(u_v)
        if isinstance(u_keys, (tuple, list)) is True:
            u_l.extend(map(lambda x: "{0}=VALUES({0})".format(x), u_keys))
        if isinstance(p1_keys, (tuple, list)) is True:
            u_l.extend(map(lambda x: "{0}={0}+1".format(x), p1_keys))
        if isinstance(concat_keys, (tuple, list)) is True:
            u_l.extend(map(lambda x: "{0}=concat({0}, ',', VALUES({0}))".format(x), concat_keys))
        if len(u_l) <= 0:
            return self.execute_insert_mul(t_name, cols, values)
        keys = cols
        sql_query = "INSERT INTO %s (%s) VALUES " % (t_name, ",".join(keys))
        if isinstance(values, (list, tuple)) is False:
            raise TypeError()
        if len(values) <= 0:
            return 0
        args = []
        for value_item in values:
            sql_query += "(" + ("%s," * len(value_item)).rstrip(",") + "),"
            args.extend(value_item)
        sql_query = sql_query.rstrip(",") + " ON DUPLICATE KEY UPDATE %s;" % ",".join(u_l)
        return self.execute(sql_query, args=args, print_sql=print_sql, auto_close=True)


class UpdateDB(SimpleDB):

    def execute_update(self, table_name, update_value=None, update_value_list=None, where_value=None, where_is_none=None,
                       where_cond=None):
        if update_value_list is None:
            update_value_list = list()
        else:
            update_value_list = list(update_value_list)
        args = []
        if update_value is not None and isinstance(update_value, dict):
            args.extend(update_value.values())
            for key in update_value.keys():
                update_value_list.append("{0}=%s".format(key))
        if len(update_value_list) <= 0:
            return 0
        sql_query = "UPDATE %s SET %s WHERE " % (table_name, ",".join(update_value_list))
        if isinstance(where_cond, tuple) or isinstance(where_cond, list):
            where_cond = list(where_cond)
        else:
            where_cond = []
        if where_value is not None:
            where_args = dict(where_value).values()
            args.extend(where_args)
            for key in dict(where_value).keys():
                where_cond.append("%s=%%s" % key)
        if isinstance(where_is_none, (list, tuple)) and len(where_is_none) > 0:
            for key in where_is_none:
                where_cond.append("%s is NULL" % key)
        sql_query += " AND ".join(where_cond) + ";"
        return self.execute(sql_query, args=args, auto_close=True)

    def execute_plus(self, table_name, *args, **kwargs):
        where_value = kwargs.pop("where_value", None)
        where_is_none = kwargs.pop("where_is_none", None)
        where_cond = kwargs.pop("where_cond", None)

        update_value_list = []
        for item in args:
            update_value_list.append("{0}={0}+1".format(item))
        for key in kwargs:
            update_value_list.append("{0}={0}+{1}".format(key, kwargs[key]))
        return self.execute_update(table_name, update_value_list=update_value_list, where_value=where_value,
                                   where_is_none=where_is_none, where_cond=where_cond)

    def execute_logic_or(self, table_name, where_value=None, where_is_none=None, where_cond=None, **kwargs):
        update_value_list = []
        for key in kwargs:
            update_value_list.append("{0}={0}|{1}".format(key, kwargs[key]))
        return self.execute_update(table_name, update_value_list=update_value_list, where_value=where_value,
                                   where_is_none=where_is_none, where_cond=where_cond)

    def execute_logic_non(self, table_name, where_value=None, where_is_none=None, where_cond=None, **kwargs):
        update_value_list = []
        for key in kwargs:
            update_value_list.append("{0}={0}&~{1}".format(key, kwargs[key]))
        return self.execute_update(table_name, update_value_list=update_value_list, where_value=where_value,
                                   where_is_none=where_is_none, where_cond=where_cond)


class DeleteDB(SimpleDB):

    def execute_delete(self, table_name, where_value=None, where_cond=None):
        where_cond, args = merge_where(where_value=where_value, where_cond=where_cond)
        if len(where_cond) <= 0:
            return 0
        sql_query = "DELETE FROM %s WHERE %s;" % (table_name,  " AND ".join(where_cond))
        return self.execute(sql_query, args, auto_close=True)


#! /usr/bin/env python
# coding: utf-8

import os
import ConfigParser
from _db import SimpleDB

__author__ = '鹛桑够'


class ConfDB(SimpleDB):
    conf_name = "mysql_app.conf"
    conf_path_environ_key = "DB_CONF_PATH"

    def __init__(self, conf_path=None, conf_dir=None, readonly=False, user=None, password=None):
        self.readonly = readonly
        if conf_path is None:
            if conf_dir is not None:
                conf_path = os.path.join(conf_dir, self.conf_name)
            elif os.environ.get(self.conf_path_environ_key) is not None:
                conf_path = os.environ.get(self.conf_path_environ_key)
            else:
                conf_path = self.conf_name
        self.conf_path = conf_path
        o = self._read_conf(conf_path, readonly)
        if user is not None and password is not None:
            o["user"] = user
            o["password"] = password
        SimpleDB.__init__(self, **o)

    @staticmethod
    def _read_conf(conf_path, readonly):
        config = ConfigParser.ConfigParser()
        config.read(conf_path)
        basic_section = "db_basic"
        host = config.get(basic_section, "host")
        db_name = config.get(basic_section, "name")
        db_port = config.getint(basic_section, "port")
        if readonly is True:
            user_section = "%s_read_user" % basic_section
        else:
            user_section = "%s_user" % basic_section
        db_user = config.get(user_section, "user")
        db_password = config.get(user_section, "password")
        return dict(host=host, db_name=db_name, port=db_port, user=db_user, password=db_password)



#! /usr/bin/env python
# coding: utf-8

#  __author__ = 'meisanggou'

try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup

import sys

if sys.version_info <= (2, 7):
    sys.stderr.write("ERROR: mysqldb-rich requires Python Version 2.7 or above.\n")
    sys.stderr.write("Your Python Version is %s.%s.%s.\n" % sys.version_info[:3])
    sys.exit(1)

name = "mysqldb-rich"
version = "3.0"
url = "https://github.com/meisanggou/mysqldb-rich"
license = "MIT"
author = "meisanggou"
short_description = "rich mysqldb"
long_description = """rich operation for mysql"""
keywords = "flask_helper"
install_requires = ["DBUtils", "mysql-python"]

setup(name=name,
      version=version,
      author=author,
      author_email="zhouheng@gene.ac",
      url=url,
      packages=["mysqldb_rich", "mysqldb_rich/db2"],
      license=license,
      description=short_description,
      long_description=long_description,
      keywords=keywords,
      install_requires=install_requires
      )
